/* ***** BEGIN LICENSE BLOCK *****
 *   Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Application Discover.
 *
 * The Initial Developer of the Original Code is Mozilla.
  *
 * Portions created by the Initial Developer are Copyright (C) 2008
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Dion Almaer <dion@almaer.com>
 *   Atul Varma <atul@mozilla.com>
 *   Jono DiCarlo <jdicarlo@mozilla.com>
 *   Blair McBride <unfocused@gmail.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 * 
 * ***** END LICENSE BLOCK ***** */

// -- Variables to be used

var Cc = Components.classes;    // -- because they are long and used now and then
var Ci = Components.interfaces;

var appdiscoverTypes = {
	'application/vnd.appcelerator-titanium-app-package+zip': 'titanium',
	'application/vnd.adobe.air-application-installer-package+zip': 'adobeair',
	'application/x-xpinstall': 'addon',
	'text/javascript': 'greasemonkey'
};

var appdiscoverStrings;


// -- Core

function setupAppDiscover() {
	appdiscoverStrings = document.getElementById("appdiscover-strings");
}

// Watch for any tags of the form <link rel="application">
function onLinkAdded(event) {
    if (event.target.rel != "application") return;
    
	showNotification(event.target.ownerDocument, event.target.href, event.target.type);    
}

function showNotification(targetDoc, applicationsUrl, mimetype) {
    // Find the <browser> which contains notifyWindow, by looking
    // through all the open windows and all the <browsers> in each.
    var wm = Cc["@mozilla.org/appshell/window-mediator;1"].getService(Ci.nsIWindowMediator);
    var enumerator = wm.getEnumerator("navigator:browser");
    var tabbrowser = null;
    var foundBrowser = null;

    while (!foundBrowser && enumerator.hasMoreElements()) {
      var win = enumerator.getNext();
      tabbrowser = win.getBrowser();
      foundBrowser = tabbrowser.getBrowserForDocument(targetDoc);
    }

    // Return the notificationBox associated with the browser.
    if (foundBrowser) {
      var box = tabbrowser.getNotificationBox(foundBrowser);
      var BOX_NAME = "appdiscover_notify_commands_available";
      var oldNotification = box.getNotificationWithValue(BOX_NAME);
      if (oldNotification)
        box.removeNotification(oldNotification);

      var buttons = [
        { accessKey: null,
          callback: function() { onInstallClick('http://google.com/'); },
          label: "Install",
          popup: null }
      ];

      box.appendNotification(
        getNotificationText(mimetype),
        BOX_NAME,
        "http://www.spreadfirefox.com/files/spreadfirefox_RCS_favicon.png",
        box.PRIORITY_INFO_MEDIUM,
        buttons
      );
    } else {
      Components.utils.reportError("Couldn't find tab for document");
    }
}

function getNotificationText(mimetype) {
	return appdiscoverStrings.getString("extensions.appdiscover.notify." + getKeyFromType(mimetype)) + ' ' + appdiscoverStrings.getString("extensions.appdiscover.notify.install");
}

function getKeyFromType(mimetype) {
	return appdiscoverTypes[mimetype] || 'default';
}

function onInstallClick(url) {
	openUrl(url);
}

function openUrl(url) {
	  var windowManager = Cc["@mozilla.org/appshell/window-mediator;1"].getService(Ci.nsIWindowMediator);
	  var browserWindow = windowManager.getMostRecentWindow("navigator:browser");
	  var browser = browserWindow.getBrowser();
	  
	  var prefService = Cc["@mozilla.org/preferences-service;1"].getService(Ci.nsIPrefBranch);
	  var openPref = prefService.getIntPref("browser.link.open_newwindow");
	  
	  //2 (default in SeaMonkey and Firefox 1.5): In a new window
	  //3 (default in Firefox 2 and above): In a new tab
	  //1 (or anything else): In the current tab or window
	  
	  if (browser.mCurrentBrowser.currentURI.spec == "about:blank" && !browser.webProgress.isLoadingDocument)
	    browserWindow.loadURI(url, null, null, false);
	  else if (openPref == 3)
	    browser.loadOneTab(url, null, null, null, false, false);
	  else if (openPref == 2)
	    window.openDialog('chrome://browser/content', '_blank', 'all,dialog=no', url, null, null, null);
	  else
	    browserWindow.loadURI(url, null, null, false);
}


// -- Setup listeners
window.addEventListener("DOMLinkAdded", onLinkAdded, false);
window.addEventListener("load", setupAppDiscover, false);
